#include "conv2bmp.h"


// LEAF Visual Novel System For MGL2
// $B0lIt%=!<%9%3!<%I$r;H$o$;$F$$$?$@$-$^$7$?!#(Bm(__)m


//---------------------------------------------------------------------------
int main(int argc, char **argv)
{
	if(argc < 2)
	{
		printf("usage: %s file\n", argv[0]);
		exit(1);
	}

	char* pName = argv[1];
	printf("convert ... %s\n", pName);


    u_char* data;
    Image* img = NULL;

	if(is_extension(pName, "LFG") == TRUE)
	{
		data = open_buf(pName);
		img = lvnsimage_load_lfg(data, NULL);
	}
	else if(is_extension(pName, "SDT") == TRUE)
	{
		data = open_buf(pName);
		img = lvnsimage_load_sdt(data, pName);
	}
	else
	{
		printf("invalid file: %s\n", pName);
		exit(1);
	}

	save_img(img, pName);

	free(data);
	image_delete(img);

	return 0;
}
//---------------------------------------------------------------------------
u_char* open_buf(const char* pName)
{
	FILE* fp = fopen(pName, "rb");

	if(fp == NULL)
	{
		printf("err filename %s\n", pName);
		exit(1);
	}

	fpos_t fsize;
	fseek(fp,0,SEEK_END);
	fgetpos(fp, &fsize);
	fseek(fp,0,SEEK_SET);

	u_char* p = (u_char*)calloc(fsize, sizeof(u_char));
	fread(p, sizeof(u_char), fsize, fp);

	return p;
}
//---------------------------------------------------------------------------
void save_img(Image* img, const char* pName)
{
/*
	printf("width: %d\n", img->width);
	printf("height: %d\n", img->height);
	printf("xoffset: %d\n", img->xoffset);
	printf("yoffset: %d\n", img->yoffset);
	printf("rwidth: %d\n", img->rwidth);
	printf("rheight: %d\n", img->rheight);
*/


	// bmp 4byte$B6-3&!#:/$G$OH/@8$7$J$$$N$G=q$+$J$/$F$b$$$$!&!&!&$1$I0l1~(B
	int pad;
	if((img->rwidth % 4) == 0)
	{
		pad = 0;
	}
	else
	{
		pad = 4 - (img->rwidth % 4);
	}

	BITMAPFILEHEADER bh;
	BITMAPINFOHEADER bi;
	RGBQUAD pal[256];
	int hsize = sizeof(bh) + sizeof(bi) + sizeof(pal);

	bh.bfType      = 0x4d42;		// 'BM'
	bh.bfSize      = hsize + (img->rwidth + pad) * img->rheight;
	bh.bfReserved1 = 0;
	bh.bfReserved2 = 0;
	bh.bfOffBits   = hsize;

	bi.biSize          = sizeof(bi);
	bi.biWidth         = img->rwidth;
	bi.biHeight        = img->rheight;
	bi.biPlanes        = 0x01;
	bi.biBitCount      = 0x08;
	bi.biCompression   = 0x00;
	bi.biSizeImage     = 0;
	bi.biXPelsPerMeter = 0;
	bi.biYPelsPerMeter = 0;
	bi.biClrUsed       = 0;
	bi.biClrImportant  = 0;


	// $B0lIt%U%!%$%k$N%Q%l%C%H$r=$@5(B
	if(strcmp("OP2_MN_W.LFG", pName) == 0)
	{
		image_fix_color(img);
	}

	// $B%m%40J30$NF)L@?'$r6/@)JQ99(B
	if(strcmp("LEAF.LFG", pName) != 0)
	{
		img->palette[15][0] = 68;
		img->palette[15][1] = 50;
		img->palette[15][2] = 68;
	}

	int i;
	for(i=0; i<256; i++)
	{
		//  RGB -> BGR
		pal[i].rgbBlue    = img->palette[i][2];
		pal[i].rgbGreen   = img->palette[i][1];
		pal[i].rgbRed     = img->palette[i][0];
		pal[i].rgbReserve = 0;
	}


	char sname[13];
	strncpy(sname, pName, 13);

	char* p = strchr(sname, '.') + 1;
	p[0] = 'B';
	p[1] = 'M';
	p[2] = 'P';

	FILE* fp = fopen(sname, "wb");
	if(fp == NULL)
	{
		printf("save err %s\n", sname);
		exit(1);
	}

	fwrite(&bh, sizeof(u_char), sizeof(bh), fp);
	fwrite(&bi, sizeof(u_char), sizeof(bi), fp);
	fwrite(&pal, sizeof(u_char), sizeof(pal), fp);


	int x, y;

	for(y=img->rheight-1; y>=0; y--)
	{
		for(x=0; x<img->rwidth; x++)
		{
			fputc(img->data[(y * img->rwidth) + x] ,fp);
		}

		for(i=0; i<pad; i++)
		{
			fputc(0x00, fp);
		}
	}

	fclose(fp);
}
//---------------------------------------------------------------------------
bool is_extension(char* pName, char* pExt)
{
	int len = strlen(pName);

	if(pName[len-4] != '.')     return FALSE;
	if(pName[len-3] != pExt[0]) return FALSE;
	if(pName[len-2] != pExt[1]) return FALSE;
	if(pName[len-1] != pExt[2]) return FALSE;

	return TRUE;
}
//---------------------------------------------------------------------------
Image* lvnsimage_load_lfg(const u_char *data, Image *over)
{
    Image *img;
    u_char *work;		/* $B:n6HNN0h(B */

    size_t size;
    int width, height, xoffset, yoffset;

    /*
     * check magic number 'LEAFCODE' 
     */
    if (strncmp(data, "LEAFCODE", 8)) {
		fprintf(stderr, "This file isn't LFG format.\n");
		exit(1);
    }

    /* geometry */
    xoffset = (data[33] << 8 | data[32]) * 8;
    yoffset = data[35] << 8 | data[34];
    width = ((data[37] << 8 | data[36]) + 1) * 8;
    height = ((data[39] << 8 | data[38]) + 1);

    if (over) {
		/* XXX no check.... */
		img = over;
		img->width = width;
		img->height = height;
		img->xoffset = xoffset;
		img->yoffset = yoffset;
		img->rwidth = width - xoffset;
		img->rheight = height - yoffset;
    } else {
		img = image_new(width, height, xoffset, yoffset);
    }

    img->palette_num = 16;
    /* read palette */
    {
		/*
		 * 4bit $BC10L(B *   RG BR GB RG BR ... 
		 */

		int i;
		const u_char *p = data + 8;
		u_char *q = (u_char *) img->palette;	/* R G B R G B R G B ... */

		int upper, lower;

		for (i = 0; i < 24; i++) {
		    upper = *p & 0xf0;
		    upper |= upper >> 4;
		    lower = *p++ & 0xf;
		    lower |= lower << 4;
		    *q++ = upper;
		    *q++ = lower;
		}
    }

    /* transparent color */
    img->transparent = data[41];

    /* image size */
    size =
	data[44] | (data[45] << 8) | (data[46] << 16) | (data[47] << 24);
    if ((work = malloc(size * sizeof *work)) == NULL) {
		perror("lfg/extract_body:");
		exit(1);
    }

    /*
     * $BE83+(B 
     */
    leafpack_lzs(data + 48, work, size);


    /*
     * convert to indexed image 
     */
    {
		int i;
		u_char *p = work;
		u_char *q = img->data;

		/*
		 * direction flag 
		 */
		int direction = data[40];

		if (!direction) {	/*
				 * VERTICAL 
				 */
		    int x = 0, y = 0;
		    for (i = 0; i < size; i++, p++) {

				q[img->rwidth * y + x] = (*p & 0x80) >> 4 |
				    (*p & 0x20) >> 3 | (*p & 0x08) >> 2 | (*p & 0x02) >> 1;
				q[img->rwidth * y + x + 1] = (*p & 0x40) >> 3 |
				    (*p & 0x10) >> 2 | (*p & 0x04) >> 1 | (*p & 0x01);
				if (++y >= img->rheight) {
				    y = 0;
				    x += 2;
				}
			}
		} else {		/*
					 * HORIZONTAL 
					 */
			for (i = 0; i < size; i++, p++) {
			*q++ = (*p & 0x80) >> 4 |
			    (*p & 0x20) >> 3 | (*p & 0x08) >> 2 | (*p & 0x02) >> 1;
			*q++ = (*p & 0x40) >> 3 |
			    (*p & 0x10) >> 2 | (*p & 0x04) >> 1 | (*p & 0x01);
		    }
		}
	}

    free(work);
    return img;
}
//---------------------------------------------------------------------------
Image* lvnsimage_load_sdt(const u_char *data, const char *name)
{
    int i, width, height;
    int si;
    size_t size;
    Image *ret;

    for (i = 0; i < 5; i++) {
      if (!strcmp(name, sdt_list[i].name)) {
	si = i;
	width = sdt_list[i].width;
	height = sdt_list[i].height;
	break;
      }
    }
    
    ret = image_new(width, height, 0, 0);

    for (i = 0; i < 16; i++) {
      ret->palette[i][0] = pal[sdt_list[si].pal][i][0];
      ret->palette[i][1] = pal[sdt_list[si].pal][i][1];
      ret->palette[i][2] = pal[sdt_list[si].pal][i][2];
    };

    size = data[0] | (data[1]<<8) | (data[2]<<16) | (data[3]<<24);
    leafpack_lzs2(data + 4, ret->data, size);

    /* $B>e2<H?E>(B */
    {
        int i, j;
        for (i=0; i<height/2; i++) {
            u_char *p = ret->data + i * width;
            u_char *q = ret->data + (height - i - 1) * width;
            for (j=0; j<width; j++, p++, q++) {
                u_char c;
                c = *p;
                *p = *q;
                *q = c;
            }
        }
    }
    return ret;
}
//---------------------------------------------------------------------------
void leafpack_lzs(const u_char * loadBuf, u_char * saveBuf, size_t size)
{
    u_char ring[0x1000];
    int i, j;
    int c, m;
    int flag;
    int pos, len;

    /*
     * initialize ring buffer 
     */
    memset(ring, 0, sizeof ring);

    /*
     * extract data 
     */
    for (i = 0, c = 0, m = 0xfee; i < size;) {

	/*
	 * flag bits, which indicates data or location 
	 */
	if (--c < 0) {
	    flag = *loadBuf++;
	    c = 7;
	}
	if (flag & 0x80) {
	    /*
	     * data 
	     */
	    saveBuf[i++] = ring[m++] = *loadBuf++;
	    m &= 0xfff;
	} else {
	    /*
	     * copy from ring buffer 
	     */

	    int data = loadBuf[0] + (loadBuf[1] << 8);
	    loadBuf += 2;

	    len = (data & 0x0f) + 3;
	    pos = data >> 4;

	    for (j = 0; j < len; j++) {
		saveBuf[i++] = ring[m++] = ring[pos++];
		m &= 0xfff;
		pos &= 0xfff;
	    }
	}
	flag = flag << 1;
    }
}
//---------------------------------------------------------------------------
void leafpack_lzs2(const u_char * pLoadBuff, u_char * pSaveBuff, size_t DataSize)
{
    u_long Index, LIndex, Len, FlagCount;
    u_short LFlag;
    u_char TextBuff[0x1011], Flag;

    /*
     * $B%F%-%9%H%P%C%U%!$N%/%j%"(B 
     */
    memset(TextBuff, 0, sizeof(TextBuff));

    /*
     * $B%F%-%9%H%P%C%U%!$N=q$-9~$_0LCV$N@_Dj(B 
     */
    Index = 0xfee;

    FlagCount = 0;
    while (DataSize > 0) {
	if (FlagCount-- > 0) {
	    Flag <<= 1;
	} else {
	    /*
	     * $B%U%i%0$N<hF@(B 
	     */
	    Flag = ~(*(pLoadBuff++));
	    FlagCount = 7;
	}

	if (Flag & 0x80) {
	    TextBuff[Index++] = *pSaveBuff++ = ~(*pLoadBuff++);
	    Index &= 0x0fff;
	    DataSize--;
	} else {
	    /*
	     * $B0JA0$K=P8=$7$?>l=j$HD9$5$N>pJs$N<hF@(B(2 byte) 
	     */
	    LFlag = ~(*(pLoadBuff) + (*(pLoadBuff + 1) << 8));
	    pLoadBuff += 2;

	    /*
	     * $BD9$5(B 
	     */
	    Len = (LFlag & 0xf) + 3;
	    DataSize -= Len;

	    /*
	     * $B0JA0=P8=$7$?0LCV(B
	     */
	    LIndex = LFlag >> 4;

	    /*
	     * $B0JA0=P8=$7$?0LCV$+$i%3%T!<(B
	     */
	    while (Len-- > 0) {
		TextBuff[Index++] = *(pSaveBuff++) = TextBuff[LIndex++];
		LIndex &= 0x0fff;
		Index &= 0x0fff;
	    }
	}
    }
}
//---------------------------------------------------------------------------
Image* image_new(int w, int h, int x, int y)
{
    Image *img;

    img = (Image *)calloc(sizeof(Image), sizeof(u_char));
    img->width = w;
    img->height = h;
    img->xoffset = x;
    img->yoffset = y;
    img->rwidth = w - x;
    img->rheight = h - y;

    img->data = (u_char *)calloc(w * h, sizeof(u_char));

//    img->hsb_data = NULL;

    return img;
}
//---------------------------------------------------------------------------
void image_delete(Image *img)
{
    if (!img)
	return;

    if (img->data) {
		free(img->data);
    }

    free(img);
}
//---------------------------------------------------------------------------
/*
 * $B!V:/!W%*!<%W%K%s%0$N$?$a$N?'Jd@5(B
 */
void image_fix_color(Image *img)
{
  int i;

  for (i = 0; i < 16; i++) {
    if (img->palette[i][0] == 0xff && img->palette[i][1] == 0xff && img->palette[i][2] == 0xff) {
      img->palette[i][0] = 0x0;
      img->palette[i][1] = 0x0;
      img->palette[i][2] = 0x0;
    }
  }
}
