#include "img.h"
#include "libmy/gbfs.h"
#include "libmy/mode3.h"
#include "libmy/fade.h"
#include "libmy/bios_arm.h"
#include "text.h"


//---------------------------------------------------------------------------
ST_IMG Img;


//---------------------------------------------------------------------------
EWRAM_CODE void ImgInit(void)
{
	_Memset(&Img, 0x00, sizeof(ST_IMG));

	Img.textFadeNum = 8;
	Img.textWaitNum = 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ImgExec(void)
{
	if(Img.isTextClear == TRUE)
	{
		ImgExecTextClear();
		return;
	}

	if(Img.isClear == TRUE)
	{
		ImgExecClear();
		return;
	}

	if(Img.isUpdate == TRUE)
	{
		ImgExecUpdate();
		return;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void ImgExecTextClear(void)
{
	switch(Img.step)
	{
	case 0:
		TextSetUnDisp();
		TextSetUnDispCur();
		FadeSetDrakOut2(1, Img.textFadeNum);

		Img.step++;
		break;

	case 1:
		FadeExecOut();

		if(FadeIsEnd() == TRUE)
		{
			Img.isText = FALSE;

			Img.step = 0;
			Img.isTextClear = FALSE;
		}
		break;
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void ImgExecClear(void)
{
	// TODO GtFNg

	switch(Img.clearType)
	{
	case IMG_EFFECT_TEXT:
		Img.isClear = FALSE;
		break;

	default:
	case IMG_EFFECT_FADE_BLACK:
		switch(Img.step)
		{
		case 0:
			FadeSetDrakIn(Img.clearNum);
			Img.step++;
			break;

		case 1:
			FadeExecIn();

			if(FadeIsEnd() == TRUE)
			{
				Mode3DrawFullFill(RGB5(0, 0, 0));
				Img.step++;
			}
			break;

		case 2:
			FadeSetClear();

			Img.step = 0;
			Img.isClear = FALSE;
			break;
		}
		break;

	case IMG_EFFECT_FADE_WHITE:
		switch(Img.step)
		{
		case 0:
			FadeSetLightIn(Img.clearNum);
			Img.step++;
			break;

		case 1:
			FadeExecIn();

			if(FadeIsEnd() == TRUE)
			{
				Mode3DrawFullFill(RGB5(31, 31, 31));
				Img.step++;
			}
			break;

		case 2:
			FadeSetClear();

			Img.step = 0;
			Img.isClear = FALSE;
			break;
		}
		break;
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void ImgExecUpdate(void)
{
	// TODO GtFNg
	u16 i;

	switch(Img.updateType)
	{
	case IMG_EFFECT_TEXT:
		switch(Img.step)
		{
		case 0:
			FadeSetDrakIn2(1, Img.textFadeNum);
			Img.step++;
			break;

		case 1:
			FadeExecIn();

			if(FadeIsEnd() == TRUE)
			{
				TextSetDisp();

				Img.isText = TRUE;
				Img.step = 0;
				Img.isUpdate = FALSE;
			}
			break;
		}
		break;

	case IMG_EFFECT_NORMAL:
	case IMG_EFFECT_FADE_MASK:
		ImgExecBg();
		ImgExecChr();
		Img.isUpdate = FALSE;
		break;

	default:
	case IMG_EFFECT_FADE_BLACK:
		switch(Img.step)
		{
		case 0:
			FadeSetDrakOut(Img.updateNum);
			ImgExecBg();
			ImgExecChr();

			Img.step++;
			break;

		case 1:
			FadeExecOut();

			if(FadeIsEnd() == TRUE)
			{
				Img.step = 0;
				Img.isUpdate = FALSE;
			}
			break;
		}
		break;

	case IMG_EFFECT_FADE_WHITE:
		switch(Img.step)
		{
		case 0:
			FadeSetLightOut(Img.updateNum);
			ImgExecBg();
			ImgExecChr();

			Img.step++;
			break;

		case 1:
			FadeExecOut();

			if(FadeIsEnd() == TRUE)
			{
				Img.step = 0;
				Img.isUpdate = FALSE;
			}
			break;
		}
		break;

	// Wave ^CgSp
	case IMG_EFFECT_SPECIAL0:
		switch(Img.step)
		{
		case 0:
			Img.updateNum = 0;
			Img.step++;
			break;

		case 1:
			if(Img.updateNum != 44 && Img.updateNum != 48 && Img.updateNum != 51)
			{
				for(i=0; i<240; i++)
				{
					Mode3DrawPsetVram(i,     Img.updateNum, RGB5(0, 0, 0));
					Mode3DrawPsetVram(i, 159-Img.updateNum, RGB5(0, 0, 0));
				}
			}
			Img.updateNum++;

			if(Img.updateNum >= 53)
			{
				Img.step = 0;
				Img.isUpdate = FALSE;
			}
			break;
		}
		break;

	// Aj[Vpi & j
	case IMG_EFFECT_SPECIAL1:
		switch(Img.step)
		{
		case 0:
			FadeSetDrakOut(0);

			Img.updateNum = 0;
			Img.step++;
			break;

		case 1:
			if(FadeIsEnd() == FALSE)
			{
				FadeExecOut();
			}

			ImgSetBgClear();
			ImgExecBg();

			ImgSetBg4("OP1W.img", 164-Img.updateNum, 0);	// 
			ImgSetBg4("OP2W.img", 0, Img.updateNum-64);		// 
			ImgExecBg();

			Img.updateNum++;

			if(Img.updateNum > 48)
			{
				FadeSetDrakIn(0);
				Img.step++;
			}
			break;

		case 2:
			ImgSetBgClear();
			ImgExecBg();

			ImgSetBg4("OP1W.img", 164-Img.updateNum, 0);
			ImgSetBg4("OP2W.img", 0, Img.updateNum-64);
			ImgExecBg();

			Img.updateNum++;

			FadeExecIn();

			if(FadeIsEnd() == TRUE)
			{
				Mode3DrawFullFill(RGB5(0, 0, 0));
				Img.step++;
			}
			break;

		case 3:
			FadeSetClear();

			Img.step = 0;
			Img.isUpdate = FALSE;
			break;
		}
		break;

	// Aj[Vpi & j
	case IMG_EFFECT_SPECIAL2:
		switch(Img.step)
		{
		case 0:
			FadeSetDrakOut(0);

			Img.updateNum = 0;
			Img.step++;
			break;

		case 1:
			if(FadeIsEnd() == FALSE)
			{
				FadeExecOut();
			}

			ImgSetBgClear();
			ImgExecBg();

			ImgSetBg4("OP3W.img", 73, Img.updateNum-64);	// 
			ImgSetBg4("OP4W.img", 48-Img.updateNum, 0);		// 
			ImgExecBg();

			Img.updateNum++;

			if(Img.updateNum > 48)
			{
				FadeSetDrakIn(0);
				Img.step++;
			}
			break;

		case 2:
			ImgSetBgClear();
			ImgExecBg();

			ImgSetBg4("OP3W.img", 73, Img.updateNum-64);
			ImgSetBg4("OP4W.img", 48-Img.updateNum, 0);
			ImgExecBg();

			Img.updateNum++;

			FadeExecIn();

			if(FadeIsEnd() == TRUE)
			{
				Mode3DrawFullFill(RGB5(0, 0, 0));
				Img.step++;
			}
			break;

		case 3:
			FadeSetClear();

			Img.step = 0;
			Img.isUpdate = FALSE;
			break;
		}
		break;

	// ^Cg\p
	case IMG_EFFECT_SPECIAL3:
		ImgSetBg3("V39.img", 0, 0);
		ImgSetBg3("TITLE.img", 82, 8);
		ImgSetBg3("TITLE2.img", 81, 93);

		Img.isUpdate = FALSE;
		break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void ImgExecBg(void)
{
	_ASSERT(Img.bg.isDraw == TRUE);


	switch(Img.bg.type)
	{
	case IMG_BG_TYPE_BACK:
	case IMG_BG_TYPE_VISUAL:
	case IMG_BG_TYPE_HCG:
		if(Img.bg.cx == SCREEN_WIDTH && Img.bg.cy == SCREEN_HEIGHT)
		{
			Mode3DrawFull(Img.bg.pDat);
		}
		else
		{
			Mode3DrawImg(Img.bg.sx, Img.bg.sy, Img.bg.cx, Img.bg.cy, Img.bg.pDat);
		}
		break;

	case IMG_BG_TYPE_BLACK:
		Mode3DrawFullFill(RGB5(0, 0, 0));
		break;

	case IMG_BG_TYPE_WHITE:
		Mode3DrawFullFill(RGB5(31, 31, 31));
		break;

	case IMG_BG_TYPE_SPECIAL:
		Mode3SetDraw(TRUE);
		break;

	default:
		SystemError("[Err] ImgExecBg Img.bg.type=%d\n", Img.bg.type);
		break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void ImgExecChr(void)
{
	u16 i;

	for(i=0; i<3; i++)
	{
		if(Img.chr[i].isDraw == FALSE)
		{
			continue;
		}

		Mode3DrawImgMask(Img.chr[i].sx, Img.chr[i].sy, Img.chr[i].cx, Img.chr[i].cy, Img.chr[i].pDat);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void ImgSetBg(char* pStr)
{
	ST_IMG_HEADER* h = (ST_IMG_HEADER*)GbfsGetSafePointer(pStr);

	_Strncpy(Img.bg.fname, pStr, IMG_MAX_FILE_LEN);
	Img.bg.sx     = (SCREEN_WIDTH  / 2) - (h->cx / 2);
	Img.bg.sy     = (SCREEN_HEIGHT / 2) - (h->cy / 2);
	Img.bg.cx     = h->cx;
	Img.bg.cy     = h->cy;
	Img.bg.pDat   = (u16*)(h + 1);
	Img.bg.type   = IMG_BG_TYPE_BACK;
	Img.bg.isDraw = TRUE;

	ImgSetChrClear(IMG_CHR_TYPE_ALL);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ImgSetBg2(u16 no)
{
	if(no == 0)
	{
		ImgSetBgClear();
		return;
	}

	char buf[64];
	_Sprintf(buf, IMG_BG_STR, no);

	ImgSetBg(buf);
}
//---------------------------------------------------------------------------
// ^Cgp
EWRAM_CODE void ImgSetBg3(char* pStr, s16 sx, s16 sy)
{
	ImgSetBg(pStr);
	Img.bg.sx = sx;
	Img.bg.sy = sy;

	// obt@ɂ̂ݏ݂܂
	ImgExecBg();
	Mode3SetDraw(FALSE);

	Img.bg.type = IMG_BG_TYPE_SPECIAL;
}
//---------------------------------------------------------------------------
// I[vjÕAj[Vp
EWRAM_CODE void ImgSetBg4(char* pStr, s16 sx, s16 sy)
{
	ImgSetBg(pStr);
	Img.bg.sx = sx;
	Img.bg.sy = sy;
	Img.bg.type = IMG_BG_TYPE_SPECIAL;

	// ݂͂`FbN
	Mode3DrawImgClip(Img.bg.sx, Img.bg.sy, Img.bg.cx, Img.bg.cy, Img.bg.pDat);
	Mode3SetDraw(FALSE);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ImgSetBgV(u16 no)
{
	char buf[64];
	_Sprintf(buf, IMG_VIS_STR, no);

	ImgSetBg(buf);
	Img.bg.type = IMG_BG_TYPE_VISUAL;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ImgSetBgH(u16 no)
{
	char buf[64];
	_Sprintf(buf, IMG_HVS_STR, no);

	ImgSetBg(buf);
	Img.bg.type = IMG_BG_TYPE_HCG;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ImgSetBgClear(void)
{
	Img.bg.type = IMG_BG_TYPE_BLACK;
	Img.bg.isDraw = TRUE;

	ImgSetChrClear(IMG_CHR_TYPE_ALL);
}
//---------------------------------------------------------------------------
// S̒܁@p
EWRAM_CODE void ImgSetBgClow(char* pStr)
{
	ImgSetBg(pStr);

	Img.bg.sx = 0;
	Img.bg.sy = SCREEN_HEIGHT - Img.bg.cy;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ImgSetChr(char* pStr, u16 type)
{
	ST_IMG_HEADER* h = (ST_IMG_HEADER*)GbfsGetSafePointer(pStr);


	s16 sy = SCREEN_HEIGHT - h->cy;
	s16 sx;

	if(type == IMG_CHR_TYPE_LEFT)
	{
		sx = 10;
	}
	else if(type == IMG_CHR_TYPE_RIGHT)
	{
		sx = SCREEN_WIDTH - h->cx - 10;
	}
	else
	{
		sx = (SCREEN_WIDTH / 2) - (h->cx / 2);
	}

	_Strncpy(Img.bg.fname, pStr, IMG_MAX_FILE_LEN);
	Img.chr[type].sx     = sx;
	Img.chr[type].sy     = sy;
	Img.chr[type].cx     = h->cx;
	Img.chr[type].cy     = h->cy;
	Img.chr[type].pDat   = (u16*)(h + 1);
	Img.chr[type].type   = type;
	Img.chr[type].isDraw = TRUE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ImgSetChr2(u16 no, u16 type)
{
	if(no == 0xff)
	{
		ImgSetChrClear(type);
		return;
	}


	char buf[64];
	_Sprintf(buf, IMG_CHR_STR, no);

	ImgSetChr(buf, type);


	// O

	// Ḧ
	if(no >= 0x70 && no <= 0x7b)
	{
		Img.chr[type].sx = 0;
	}

	// H̏
	if(no >= 0x90 && no <= 0x98)
	{
		Img.chr[type].sx = SCREEN_WIDTH - Img.chr[type].cx;
	}

	// Qׂ
	if(no == 0x58)
	{
		Img.chr[type].sx = 116;
		Img.chr[type].sy = 60;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void ImgSetChrClear(u16 type)
{
	if(type == IMG_CHR_TYPE_ALL)
	{
		Img.chr[0].isDraw = FALSE;
		Img.chr[1].isDraw = FALSE;
		Img.chr[2].isDraw = FALSE;
	}
	else
	{
		Img.chr[type].isDraw = FALSE;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void ImgSetEffectClear(u16 type)
{
	ImgSetEffectClear2(type, 2);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ImgSetEffectClear2(u16 type, u16 num)
{
	Img.isTextClear = Img.isText;

	Img.clearType = type;
	Img.clearNum = num;

	Img.step = 0;
	Img.isClear = TRUE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ImgSetEffectUpdate(u16 type)
{
	ImgSetEffectUpdate2(type, 2);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ImgSetEffectUpdate2(u16 type, u16 num)
{
	if(type == IMG_EFFECT_TEXT && Img.isText == TRUE)
	{
		return;
	}
	Img.isTextClear = Img.isText;

	Img.updateType = type;
	Img.updateNum = num;

	Img.step = 0;
	Img.isUpdate = TRUE;
}
//---------------------------------------------------------------------------
EWRAM_CODE u16 ImgGetTextFadeNum(void)
{
	return Img.textFadeNum;
}
//---------------------------------------------------------------------------
EWRAM_CODE u16 ImgGetTextWaitNum(void)
{
	return Img.textWaitNum;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ImgSetTextFadeNum(u16 num)
{
	Img.textFadeNum = num;
	FadeSetDirect(num);		// IɕύX܂
}
//---------------------------------------------------------------------------
EWRAM_CODE void ImgSetTextWaitNum(u16 num)
{
	Img.textWaitNum = num;
}
//---------------------------------------------------------------------------
EWRAM_CODE bool ImgIsEnd(void)
{
	return (Img.isTextClear == TRUE || Img.isClear == TRUE || Img.isUpdate == TRUE) ? FALSE : TRUE;
}
//---------------------------------------------------------------------------
EWRAM_CODE bool ImgIsText(void)
{
	return Img.isText;
}
