#include "script.h"
#include "script2.h"
#include "history.h"
#include "libmy/gbfs.h"
#include "libmy/key.h"
#include "img.h"
#include "anime.h"
#include "menu.h"
#include "siori.h"


//---------------------------------------------------------------------------
u8 ScriptEffectTable[SCRIPT_MAX_EFFECT_CNT] = {
	IMG_EFFECT_FADE_BLACK,
	IMG_EFFECT_GURUGURU,
	IMG_EFFECT_FADE_MASK,
	IMG_EFFECT_FADE_SQUARE,
	IMG_EFFECT_WIPE_SQUARE_LTOR,
	IMG_EFFECT_FADE_MASK,
	IMG_EFFECT_WIPE_TTOB,
	IMG_EFFECT_WIPE_LTOR,
	IMG_EFFECT_WIPE_MASK_LTOR,
	IMG_EFFECT_VERTCOMPOSITION,

	IMG_EFFECT_SLIDE_LTOR,
	IMG_EFFECT_NORMAL,
	IMG_EFFECT_FADE_MASK,
	IMG_EFFECT_CIRCLE_SHRINK,
	IMG_EFFECT_NONE,
	IMG_EFFECT_NONE,
	IMG_EFFECT_NONE,
	IMG_EFFECT_LEFT_SCROLL,
	IMG_EFFECT_FADE_BLACK,
	IMG_EFFECT_RAND_RASTER,
};


//---------------------------------------------------------------------------
ST_SCRIPT Script;


//---------------------------------------------------------------------------
EWRAM_CODE void ScriptInit(void)
{
	_Memset(&Script, 0x00, sizeof(ST_SCRIPT));
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScriptInitFlag(void)
{
	_Memset(&Script.flag, 0x00, sizeof(Script.flag));

	// Z[uf[^ꍇAꕔ̃tOiflag, flagEndj㏑܂
	SioriLoadFlag();
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScriptExec(void)
{
	if(ImgIsEnd() == FALSE)
	{
		return;
	}

	if(TextIsEnd() == FALSE)
	{
		return;
	}


	switch(Script.act)
	{
	case SCRIPT_ACT_EVENT:
	case SCRIPT_ACT_MSG:
		ScriptExecParse();
		break;

	case SCRIPT_ACT_ANIME:
		ScriptExecAnime();
		break;

	case SCRIPT_ACT_WAIT_PAGE:
		ScriptExecWaitPage();
		break;

	case SCRIPT_ACT_WAIT_KEY:
		ScriptExecWaitKey();
		break;

	case SCRIPT_ACT_HISTORY:
		ScriptExecHistory();
		break;

	case SCRIPT_ACT_SELECT:
		ScriptExecSelect();
		break;

	case SCRIPT_ACT_MENU:
		ScriptExecMenu();
		break;

	default:
		SystemError("[Err] ScriptExec Script.act = %d\n", Script.act);
		break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScriptExecAnime(void)
{
	AnimeExec();

	if(AnimeIsEnd() == TRUE)
	{
		ScriptSetAct(SCRIPT_ACT_MSG);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScriptExecWaitPage(void)
{
	if(ScriptCalcKey() == TRUE)
	{
		TextSetNewPage();

		ScriptSetAct(SCRIPT_ACT_MSG);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScriptExecWaitKey(void)
{
	if(ScriptCalcKey() == TRUE)
	{
		ScriptSetAct(SCRIPT_ACT_MSG);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScriptExecHistory(void)
{
	switch(Script.actStep)
	{
	case 0:
		HistoryInitAct();
		Script.actStep++;
		break;

	case 1:
		HistoryExec();

		if(HistoryIsEnd() == TRUE)
		{
			TextSetRestore();
			ScriptPopAct();
		}
		break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScriptExecMenu(void)
{
	switch(Script.actStep)
	{
	case 0:
		MenuSetSystem();
		Script.actStep++;
		break;

	case 1:
		MenuExec();

		if(MenuIsEnd() == TRUE)
		{
			ScriptPopAct();
		}
		break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScriptExecSelect(void)
{
	u16 trg = KeyGetTrg();
	s16 i;

	switch(Script.actStep)
	{
	case 0:
		// ₢ƁAڐ擾܂
		Script.selMsg = *Script.pCur++;
		Script.selCnt = *Script.pCur++;
		Script.selNum = -1;

		for(i=0; i<Script.selCnt; i++)
		{
			Script.selItem[i] = *Script.pCur++;
			Script.selJump[i] = *Script.pCur++;
		}

		TRACEOUT("[I̐: %d]\n", Script.selCnt);
		Script.actStep++;
		break;

	case 1:
		// ₢\܂
		ScriptExecSelectSub(Script.selMsg, FALSE, FALSE);
		ImgSetEffectUpdate(IMG_EFFECT_TEXT);

		Script.actStep++;
		break;

	case 2:
		// ͑҂
		if(ScriptCalcKey() == TRUE)
		{
			TextSetNewPage();
			Script.actStep++;
		}
		break;

	case 3:
		// ڂ\܂
		TextSetClearXY();

		for(i=0; i<Script.selCnt; i++)
		{
			if(i == Script.selNum)
			{
				TextSetImgWhite();
			}
			else
			{
				TextSetImgGray();
			}

			ScriptExecSelectSub(Script.selItem[i], TRUE, FALSE);
			TextSetLf();
		}
		TextSetImgWhite();
		Script.actStep++;
		break;

	case 4:
		// I
		if((trg & KEY_A) && Script.selNum != -1)
		{
			Script.actStep++;
		}
		else if((trg & KEY_LEFT) && (HistoryIsEmpty() == FALSE))
		{
			Script.actStep = 3;
			ScriptPushAct(SCRIPT_ACT_HISTORY);
		}
		else if(trg & KEY_B)
		{
			Script.actStep = 3;
			ScriptPushAct(SCRIPT_ACT_MENU);
		}
		else if((trg & KEY_UP) && Script.selNum > 0)
		{
			Script.actStep = 3;
			Script.selNum--;
		}
		else if((trg & KEY_DOWN) && (Script.selNum + 1) < Script.selCnt)
		{
			Script.actStep = 3;
			Script.selNum++;
		}
		break;

	case 5:
		// II
		TextSetClearXY();
		ScriptExecSelectSub(Script.selItem[Script.selNum], TRUE, TRUE);
		TextSetClear();

		Script.isNext = FALSE;
		Script.pCur += Script.selJump[Script.selNum];

		ScriptSetAct(SCRIPT_ACT_EVENT);
		break;
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void ScriptExecSelectSub(u8 msgNo, bool isSel, bool isBuf)
{
	u8* pCur = Script.pMsg + *((u16*)Script.pMsg + (msgNo + 1));

	u16 buf[512] ALIGN(4);
	u16 cnt = 0;

	for(;;)
	{
		if(*pCur < 0x20)
		{
			break;
		}

		if(*pCur == 0xb9 || *pCur == 0xb6)
		{
			pCur += 2;
		}
		else
		{
			pCur++;
		}
	}

	do
	{
		buf[cnt++] = (*pCur << 8) | (*(pCur + 1));
		pCur += 2;

		// s`FbN
		if(*pCur == 0xb0)
		{
			pCur++;
		}

	} while(*pCur < 0x20);

	_ASSERT(cnt < 512 && "ScriptExecSelectSub buf overflow");


	if(isSel == TRUE)
	{
		TextSetBufWorkNoWait(buf, cnt, isBuf);
	}
	else
	{
		TextSetBufWork(buf, cnt);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScriptSetAct(u16 act)
{
	Script.act = act;
	Script.actStep = 0;

	Script.actTmp = 0;
	Script.actStepTmp = 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScriptPushAct(u16 act)
{
	_ASSERT(Script.actTmp == 0);
	_ASSERT(Script.actStepTmp == 0);

	Script.actTmp = Script.act;
	Script.actStepTmp = Script.actStep;

	Script.act = act;
	Script.actStep = 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScriptPopAct(void)
{
	_ASSERT(Script.actTmp != 0);

	Script.act = Script.actTmp;
	Script.actStep = Script.actStepTmp;

	Script.actTmp = 0;
	Script.actStepTmp = 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScriptSetNext(bool is)
{
	Script.isNext = is;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScriptSetScn(u16 scnNo)
{
	char buf[64];
	_Sprintf(buf, SCRIPT_SCN_STR, scnNo);

	ST_SCRIPT_HEADER* p = (ST_SCRIPT_HEADER*)GbfsGetSafePointer(buf);

	Script.scnNo     = scnNo;
	Script.pEvent    = (u8*)p + p->eventOffset;
	Script.pMsg      = (u8*)p + p->msgOffset;
	Script.eventSize = p->eventSize;
	Script.msgSize   = p->msgSize;
	Script.pCur      = NULL;

	TRACEOUT("[XNvg[h: %03d.SCN 0x%x 0x%x]\n", scnNo, Script.pEvent, Script.pMsg);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScriptSetEvent(u16 eventNo)
{
	_ASSERT(Script.pEvent != NULL);

	Script.pCur = Script.pEvent + *((u16*)Script.pEvent + (eventNo + 1));
	ScriptSetAct(SCRIPT_ACT_EVENT);

	TRACEOUT("[Cxg[h: no.%d 0x%x]\n", eventNo + 1, Script.pCur);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScriptSetMsg(u16 msgNo)
{
	_ASSERT(Script.pMsg != NULL);

	Script.pCur = Script.pMsg + *((u16*)Script.pMsg + (msgNo + 1));
	ScriptSetAct(SCRIPT_ACT_MSG);

	TRACEOUT("[bZ[W[h: no.%d 0x%x]\n", msgNo + 1, Script.pCur);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScriptSetFlagEnd(u8 no, u8 val)
{
/*
	0.ߐ^
	1.ߍK
	2.ߎE
	3.
	4.K
	5.E
	6.
	7.
	8.K
	9.^
	10.쐳
	11.K
	12.
	13.Rs[
	14.ʂ蔲
	15.K`s
	16.؉Ƃ̐H
*/

	_ASSERT(no < SCRIPT_MAX_FLAG_END_CNT);
	Script.flagEnd[no] = val;


	// SNA`FbN

	u16 i;

	for(i=0; i<SCRIPT_MAX_FLAG_END_CNT; i++)
	{
		if(Script.flagEnd[i] != 0x01)
		{
			break;
		}
	}

	if(i == SCRIPT_MAX_FLAG_END_CNT)
	{
		ScriptSetFlag(0x1b, 1);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScriptSetFlag(u8 no, u8 val)
{
	_ASSERT(no < SCRIPT_MAX_FLAG_CNT);
	Script.flag[no] = val;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScriptAddFlag(u8 no, u8 val)
{
	_ASSERT(no < SCRIPT_MAX_FLAG_CNT);
	Script.flag[no] += val;
}
//---------------------------------------------------------------------------
EWRAM_CODE u8 ScriptGetFlag(u8 no)
{
	_ASSERT(no < SCRIPT_MAX_FLAG_CNT);
	return Script.flag[no];
}
//---------------------------------------------------------------------------
// ViItO̕ϊ
EWRAM_CODE u8 ScriptGetMapFlag(u8 no)
{
/*
	0x00: Ɖb
	0x01: Ɛlł͂Ȃ
	0x02: ߂^
	0x03: Ǝɂނ
	0x04: ViIx@ɓdb
	0x05: OrA𔃂
	0x06: 𔃂
	0x07: 킪点Ȃ
	0x08: ܂킽
	0x09: gp?

	0x0a - 0x13: p? (GfBOAXNvgŏ)

	0x14: BADGh
	0x15: ߃GfBO
	0x16:  Happy 
	0x17:  BAD 
	0x18:  HAPPY  
	0x19: Gh
	0x1a: Gh

	0x1b: SGfBOǂiXNvgł0x73j
*/

	if(no == 0x73)
	{
		return 0x1b;
	}

	return no;
}
//---------------------------------------------------------------------------
// wi摜 p[^ϊ
EWRAM_CODE u8 ScriptGetMapBg(u8 no)
{
	if(no == 254)
	{
		return 0;
	}

	return Mod(no, 0x32);
}
//---------------------------------------------------------------------------
// GtFNgp[^̕ϊ
EWRAM_CODE u8 ScriptGetMapEffect(u8 no)
{
	_ASSERT(no < SCRIPT_MAX_EFFECT_CNT);

	return ScriptEffectTable[no];
}
//---------------------------------------------------------------------------
// BGMp[^̕ϊ
EWRAM_CODE u8 ScriptGetMapBgm(u8 no)
{
	return no + 2;
}
//---------------------------------------------------------------------------
EWRAM_CODE bool ScriptCalcKey()
{
	TextSetDispCur();

	u16 cnt = KeyGetCnt();
	u16 trg = KeyGetTrg();

	//ȗI܂ŐiށvtÕ`FbN܂
	if(Script.isNext == TRUE)
	{
		if(cnt & KEY_B)
		{
			Script.isNext = FALSE;
		}
		else
		{
			TextSetUnDispCur();
			return TRUE;
		}
	}

	if(cnt & KEY_R || trg & KEY_A || trg & KEY_DOWN)
	{
		TextSetUnDispCur();

		return TRUE;
	}
	else if((trg & KEY_LEFT) && (HistoryIsEmpty() == FALSE))
	{
		TextSetUnDispCur();
		ScriptPushAct(SCRIPT_ACT_HISTORY);

		return FALSE;
	}
	else if(trg & KEY_B)
	{
		TextSetUnDispCur();
		ScriptPushAct(SCRIPT_ACT_MENU);

		return FALSE;
	}

	return FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE bool ScriptIsEnd(void)
{
	return (Script.act == SCRIPT_ACT_END) ? TRUE : FALSE;
}
