/*
 * LEAF Visual Novel System For X
 * (c) Copyright 1999,2000 Go Watanabe mailto:go@denpa.org
 * All rights reserverd.
 *
 * ORIGINAL LVNS (c) Copyright 1996-1999 LEAF/AQUAPLUS Inc.
 *
 * $Id: LvnsInfo.h,v 1.1 2002/07/29 05:03:30 go Exp $
 *
 */

#ifndef __LvnsInfo_h
#define __LvnsInfo_h

#include <sys/types.h>

#ifdef DEBUG
#define dprintf(a) fprintf a
#else
#define dprintf(a)
#endif

#ifndef Bool
#define Bool int
#endif

#ifndef True
#define True 1
#endif

#ifndef False
#define False 0
#endif

#include "leafpack.h"   /* Leaf Pack File   */
#include "lvnsimage.h"  /* Leaf Image Data  */

#define VERSION "LEAF Visual Novel System for X\n"\
                " ==== XLVNS 1-2-3 Ver 3.0beta ====\n"\
                "(c) Copyright 1999,2000 Go Watanabe\n"\
                "Original LVNS (c)LEAF/AQUAPLUS\n"

#define INTERVAL 100

#ifndef XLVNSPACKDIR
#define XLVNSPACKDIR "/usr/local/share/xlvns"
#endif

#define NOCHARACTER 255

/* EUC ɤѥå JIS ɤء */
#define EucToJisPack(code)  ((((code>>8)&0x7f)-33)*94 + ((code&0x7f)-33))

#define LVNS     lvns->system_state
#define WIDTH	(lvns->system_state->width)
#define HEIGHT	(lvns->system_state->height)
#define XPOS(x,y)    ((x) * 24 + lvns->tvram[lvns->current_tvram].row[y].offset)
#define YPOS(y)      ((y) * 28 + 8)
#define R_XPOS(x, y) (((x) - lvns->tvram[lvns->current_tvram].row[y].offset)/24)
#define R_YPOS(y)    (((y) - 8) / 28)

struct LvnsInfo;

typedef enum {
    LVNS_CURSOR_PAGE,
    LVNS_CURSOR_KEY,
}LvnsCursorType;

typedef enum {
    LVNS_EFFECT_NONE,
    LVNS_EFFECT_NORMAL,            /* ñɽ */
    LVNS_EFFECT_FADE_MASK,         /* ޥե */
    LVNS_EFFECT_WIPE_TTOB,         /* 夫磻 */
    LVNS_EFFECT_WIPE_LTOR,         /* 磻 */
    LVNS_EFFECT_FADEOUT,           /* Palette ե */
    LVNS_EFFECT_WHITEOUT,          /* Palette ۥ磻ȥ */
    LVNS_EFFECT_WIPE_MASK_LTOR,    /* 磻(ޥ) */
    LVNS_EFFECT_FADE_SQUARE,       /* Ҥե */
    LVNS_EFFECT_WIPE_SQUARE_LTOR,  /* Ҥ磻 */
    LVNS_EFFECT_SLIDE_LTOR,        /* ĥ饤 */
    LVNS_EFFECT_SLANTTILE,         /* Ф첣⥶饤 */
    LVNS_EFFECT_GURUGURU,          /* 뤰 */
    LVNS_EFFECT_VERTCOMPOSITION,   /* ǤϤߤ */
    LVNS_EFFECT_CIRCLE_SHRINK,     /* 濴ˤ */
    LVNS_EFFECT_LEFT_SCROLL,       /* ˥ */
    LVNS_EFFECT_TOP_SCROLL,        /* ˥ */
    LVNS_EFFECT_RAND_RASTER,       /* ɽ */
    LVNS_EFFECT_BLOOD,             /* 줬   */
    LVNS_EFFECT_VIBRATO,           /* ̤ɤ餹 */
} LvnsEffectType;                  /* եȼ */

typedef Bool (*LvnsFunc)(struct LvnsInfo *lvns);
typedef void (*LvnsBackEffectSetFunc)(struct LvnsInfo *lvns);
typedef void (*LvnsBackEffectFunc)(struct LvnsInfo *lvns);
typedef struct {
    LvnsBackEffectSetFunc set;
    LvnsBackEffectFunc func;
} LvnsBackEffectInfo;


typedef struct {
    enum {
        LVNS_ANIM_NONE,
        LVNS_ANIM_IMAGE,
        LVNS_ANIM_IMAGE2,
        LVNS_ANIM_IMAGE_ADD,
        LVNS_ANIM_IMAGE_ADD2,
        LVNS_ANIM_DISP,
        LVNS_ANIM_SOUND,
        LVNS_ANIM_WAIT,
    } type;
    const char *name;
    int time;
    int x;
    int y;
    LvnsImage *image;
} LvnsAnimationData;    /* ñ㥢˥ */

typedef struct {
    enum {
        LVNS_SCRIPT_BG,                 /* طʥ       */
        LVNS_SCRIPT_DISP,               /* ɽ(BG) */
        LVNS_SCRIPT_DISP_VRAM,          /* ɽ         */
        LVNS_SCRIPT_CLEAR,              /* õ         */               
        LVNS_SCRIPT_CLEARDISP,          /* õɽ     */               
        LVNS_SCRIPT_MUSIC,              /* ں         */
        LVNS_SCRIPT_MUSIC_FADE,
        LVNS_SCRIPT_WAIT_MUSIC,         /* ڽλԤ     */
        LVNS_SCRIPT_ANIM,
        LVNS_SCRIPT_KEY,
        LVNS_SCRIPT_TEXT,
        LVNS_SCRIPT_TEXT_CENTER,
        LVNS_SCRIPT_WAIT,               /* Ԥ         */
        LVNS_SCRIPT_TIMER_INIT,         /* Ԥ֥޽ */
        LVNS_SCRIPT_TIMER_WAIT,         /* ַвޤԤ */
        LVNS_SCRIPT_FUNC,               /* ؿƤӽФ         */
        LVNS_SCRIPT_KEY_JUMP,
        LVNS_SCRIPT_EXIT,
        LVNS_SCRIPT_END
    } type;
    void *data0;
    void *data1;
    void *data2;
} LvnsScriptData;       /* OP/ED ץȽ */

typedef struct LvnsScript {
    LvnsScriptData *data;       /* ץȥǡ       */
    int cur;                    /* ¹ԥ           */
    int state;                  /* ơ               */
} LvnsScript;

typedef int LvnsScriptFunc(struct LvnsInfo *lvns, LvnsScript *scr, void *param1, void *param2);

typedef struct {
    int scn;   /* ʥꥪֹ */
    int text;  /* ƥֹ */
} LvnsHistoryData;      /* ۥ⡼ */

typedef struct {
    char *command;
    int enable;
} LvnsCommandInfo;   /*  */

typedef struct {
    int width;              /* ΰΥ */
    int height;             
    const char *leafpack_name;   /* ǡѥѥåե̾   */
    const char *scnpack_name;    /* ʥꥪѥѥåե̾ */
    const char *scn_name;         /* ʥꥪե١̾ */
    const char *fonttable_name;  /* ɥСȥơ֥̾ */
    void (*margeCharacter)(struct LvnsInfo *lvns);   /* 饯 */
    void (*clearScreen)(struct LvnsInfo *lvns);      /*  VRAM ξõ */
    void (*loadBG)(struct LvnsInfo *lvns, int no);   /* طɤ߹ */
    LvnsCommandInfo* (*getCommandList)(struct LvnsInfo *lvns);  /*  */
    void (*execCommand)(struct LvnsInfo *lvns, const char *command);     /*  */
    void (*drawChar)(struct LvnsInfo *lvns, int x, int y, int, int);

    int cursor_key;
    int cursor_page;

} LvnsSystemState;

typedef struct LvnsInfo {
    int type;               /* ƥμ */
    void *system_depend;                /* ưƥ¸ */

    /* ------------------------------------------------------------ */

    LvnsSystemState *system_state;     /* ƥॹơ */

    char *data_path;       /* ǡܥѥ */
    char *savedata_path;   /* ǡ¸ */
 
    LeafPack *leafpack;     /* ǡѥѥåե */
    LeafPack *scnpack;      /* ʥꥪѥѥåե */
    u_char *leaf_font_data; /* Leaf FONT Υǡ   */

    u_char *leaf_to_euc;    /* ɥСȥơ֥ Leaf -> EUC  */
    u_short *jis_to_leaf;   /* ɥСȥơ֥ JIS Pack -> Leaf */

    /* ߤξ ǡ¸ */
    enum bgtype {
        LVNS_VISUAL,
        LVNS_HCG,
        LVNS_BACKGROUND
    } bg_type;
    int bg_no;
    int left_no;
    int right_no;
    int center_no;

    Bool savepoint_flag;
    struct SavePoint{
        enum bgtype bg_type;
        int bg_no;
        int left_no;     /* Ω¸ */
        int right_no;    
        int center_no;
        int scn;         /* ʥꥪֹ¸ */
        int blk;         /* ʥꥪ֥å¸       */
        long scn_offset; /* ʥꥪǡ¸ѥ */
        int current_music;
    } savepoint;
    struct SavePoint selectpoint;


    /* ------------------------------------------------------ */
    /* ƥȥ쥤 */

#define TEXT_WIDTH  25
#define TEXT_HEIGHT 16

    struct {
        /* ƥȥ쥤ѲVRAM */
	
	struct TextVramLine {
	    int offset;
	    struct TextVram {
		int changed;    /* 줿ɤ */
		int code;       /* ʸΥ       */
		int attribute;  /* ʸ°         */
	    } column[TEXT_WIDTH];
	} row[TEXT_HEIGHT];

        int cur_x;
        int cur_y;
        int o_cur_x;
        int o_cur_y;
    } tvram[2];
    int current_tvram;

    /* ξ */
    int cursor_state;
    
    /* ƥȥξ */
    int text_cursor_state;     

    /* ξ */
    Bool select_mode;
    int select_num;
    struct {
	int no;
	int offset;
    } select_info[12];
    
    int text_attr;

    /* ---------------------------------------------------------- */
    /* ʥꥪѡ */

    int start_scn_num;    /* ưʥꥪֹ(for DEBUG) */

    u_char *scn_data;     /* ʥꥪǡ             */
    u_char *scn_cur_head; /* ʥꥪǡƬ         */
    u_char *scn_cur;      /* ʥꥪǡ     */

    u_char *scn_text;     /* ʥꥪƥ           */
    u_char *scn_text_cur; /* ƥ         */
    size_t  scn_length;

    int scn_current;      /* ߤΥʥꥪֹ   */
    int blk_current;      /* ߤΥʥꥪ֥å */

    void (*scn_func)(struct LvnsInfo *);           /* ʥꥪ󥸥 */
    void (*menu_init_func)(struct LvnsInfo *);     /* ˥塼 */
    Bool (*menu_func)(struct LvnsInfo *);          /* ˥塼   */

    /*  */
    enum LvnsState {
        LVNS_STOP,
        LVNS_SCENARIO,          /* ʥꥪѡ     */
        LVNS_WAIT_DARKEN,       /* ̤Ť       */
        LVNS_WAIT_LIGHTEN,      /* ̤뤯     */
        LVNS_WAIT_CLEARDISP,    /* õɽ           */
        LVNS_WAIT_CLEAR,        /* õեȽ */
        LVNS_WAIT_DISP,         /* ɽեȽ */
        LVNS_WAIT_DISP_KEY,     /* ɽեȽ */
        LVNS_WAIT_KEY,          /* Ԥ     */
        LVNS_WAIT_KEY2,         /* Ԥ(ɽ̵) */
        LVNS_WAIT_PAGE,         /* ڡԤ     */
        LVNS_WAIT_SELECT,       /* Ԥ         */
        LVNS_WAIT_MUSIC,        /* BGM λԤ     */
        LVNS_WAIT_MUSIC_FADE,   /* BGM FADE Ԥ */
        LVNS_WAIT_SOUND,        /* ɺλԤ   */
        LVNS_WAIT_ANIM,         /* ˥᡼λԤ */
        LVNS_WAIT_FUNCTION,     /* ѽƤӽФ */
        LVNS_MENU,              /* ˥塼     */
        LVNS_MENU2,             /* ˥塼 (󥻥̵) */
        LVNS_WAIT_MENU,         /* ˥塼Ԥ */ 
        LVNS_WAIT_MENU2,        /* ˥塼Ԥ (󥻥̵)*/ 
   } state;      

    enum LvnsState state_bak; /* ֤ΥХåå */

    Bool cleard;              /* ̾õλե饰 */
    Bool noredisp;            /* ʥꥪäʸäƤ򤷤ʤ*/
    Bool image_mode;

    /* ʥꥪ⡼ */
    enum {
        LVNS_DATA,      /* ʥꥪץ */
        LVNS_TEXT,      /* ƥȥʥꥪ   */
        LVNS_TEXT_CLEAR, /* ƥȥʥꥪ   */
        LVNS_ADDTEXT,   /* ɲåƥ       */
    } scn_mode;
    
    Bool            image_changed; /* ֥åե饰          */
    Bool            image_dark;    /* ߤΰŤ                      */
    Bool            image_touched; /* ƥȤǤ褵줿? */

    Bool            text_written;  /* ƥȤ褵Ƥ */
    Bool            text_changed;  /* ƥȤ줿     */
    Bool            text_flushed;  /* ƥȤõ줿   */

    Bool            key_clicked;   /* åե饰   */

    Bool            fast_text;    /* ƥȤФե饰       */
    Bool            fast_disp;    /* ɽեȤäȤФե饰 */

    int             char_wait_time;         /* ʸɽԤ     */
    int             char_wait_time_default; /* ɸʸɽԤ */
    
    int             wait_time;   /* Ԥֻ       */
    int             wait_scn;    /* ʥꥪԤ */

    Bool            force_fast_disp; /* ʸǤ⤹äȤФ */
    Bool            key_click_fast; /* åʸ®ɽ */
    Bool            seen_mode;    /* ʸɤȤߤʤ⡼(for DEBUG) */
    Bool            demo_mode;    /* ȥǥ⡼ */

    int             demo_cnt;     /* ǥѥ   */
    int             demo_choice;  /* ǥ˥塼 */

    Bool            interval_disp; /* 󥿡Хɽ뤫? */

    /* ------------------------------------------------------- */
    /*  */

    LvnsImage *vram_bg;    /* طʥǡݻ        */
    LvnsImage *vram;       /* طʹѲ VRAM */

    int latitude;          /* ߤβ뤵 0-255          */
    int latitude_dark;     /* ̤Ťε٤λ          */

    LvnsEffectType effect_clear; /* õե     */
    LvnsEffectType effect_disp;  /* ɽե     */
    int effect_state;            /* եѥơ */

    LvnsBackEffectInfo *effect_back;
    LvnsBackEffectInfo *effect_back_next;
    int effect_back_state;               /* طʥեѥơ */


    /* ---------------------------------------------------------- */
    /* ˥᡼ */
    LvnsAnimationData *anim_data;
    int anim_off_x;
    int anim_off_y;

    /* ---------------------------------------------------------- */
    /* ѽƤӽФ */
    LvnsFunc func;
    int func_state;

    /* -------------------------------------------------------- */
    /* ں */

    void *music_depend;                           /* Ķ¸ǡ    */
    void (*InitMusic)(void *dep);                 /* BGM         */
    void (*CloseMusic)(void *dep);

    void (*StartMusic)(void *dep, int no);        /* BGM           */
    void (*StopMusic)(void *dep);                 /* BGM           */
    void (*PauseMusic)(void *dep);                /* BGM       */
    void (*SetMusicVolume)(void *dep, int no);    /* BGMܥ塼 */
    int  (*GetMusicState)(void *dep);             /* BGMå */

    /* 롼פνϼ */
    int current_music;                            /* ߱β     */
    Bool loop_music;                              /* 롼פɤ       */
    int next_music;                               /* Υ󤫤β */
    Bool loop_next_music;                         /* 롼פɤ       */

    int current_music_volume;                     /* ߤΥܥ塼 */
    int music_fade_mode;                          /* եɥ⡼       */
    int music_fade_flag;                          /* եɤ       */

    /* -------------------------------------------------------- */
    /* ɺ */

    void *sound_depend;                              /* Ķ¸ǡ */
    void (*InitSound)(void *dep);
    void (*CloseSound)(void *dep);

    void (*LoadSound)(void *dep, const char *name);
    void (*StartSound)(void *dep);
    void (*StopSound)(void *dep);
    int  (*GetSoundState)(void *dep);

    /* 롼׽ϼ */
    int sound_loop;
    int sound_count;

} LvnsInfo;

/* LvnsText.c ƥ */
void LvnsLocate(LvnsInfo *lvns, int x, int y);
void LvnsPutChar(LvnsInfo *lvns, int c, int attr);
void LvnsPutCharNormal(LvnsInfo *lvns, int c, int attr);
void LvnsPuts(LvnsInfo *lvns, u_char *str, int attr);
void LvnsPutsCenter(LvnsInfo *lvns, int y, u_char *str, int attr);
void LvnsClearText(LvnsInfo *lvns);
void LvnsSetTextOffset(LvnsInfo *lvns, int offset);
void LvnsNewLineText(LvnsInfo *lvns);
void LvnsPageWait(LvnsInfo *lvns);
void LvnsKeyWait(LvnsInfo *lvns);

/* LvnsAnim.c ˥᡼ */
void LvnsInitAnimation(LvnsInfo *lvns, LvnsAnimationData *data);
void LvnsSetAnimation(LvnsInfo *lvns, LvnsAnimationData *data, 
                          int off_x, int off_y);
Bool LvnsAnimation(LvnsInfo *lvns);

/* LvnsMusic.c  */
void LvnsSetMusicVolume(LvnsInfo *lvns, int no);
void LvnsStartMusic(LvnsInfo *lvns, int no);
void LvnsStopMusic(LvnsInfo *lvns);
void LvnsPauseMusic(LvnsInfo *lvns);

void LvnsStartMusicLoop(LvnsInfo *lvns, int no);
void LvnsStartMusicLoop2(LvnsInfo *lvns, int no);
void LvnsSetNextMusic(LvnsInfo *lvns, int no);
void LvnsSetNextMusicLoop(LvnsInfo *lvns, int no);
void LvnsFadeMusic(LvnsInfo *lvns);
void LvnsStartNextMusic(LvnsInfo *lvns);

void LvnsLoopMusic(LvnsInfo *lvns);
void LvnsWaitMusic(LvnsInfo *lvns);
void LvnsWaitMusicFade(LvnsInfo *lvns);

/* LvnsSound.c ̲ */
void LvnsLoadSound(LvnsInfo *lvns, const char* basename, int no);
void LvnsLoadSound2(LvnsInfo *lvns, const char* name);
void LvnsStartSound(LvnsInfo *lvns, int count);
void LvnsStopSound(LvnsInfo *lvns);
void LvnsWaitSound(LvnsInfo *lvns);
void LvnsLoopSound(LvnsInfo *lvns);

/* LvnsBackEffect.c طʥե */
void LvnsSetBackEffect(LvnsInfo *lvns, LvnsBackEffectInfo *info);
void LvnsSetNextBackEffect(LvnsInfo *lvns, LvnsBackEffectInfo *info);
void LvnsBackEffectSetState(LvnsInfo *lvns);
void LvnsBackEffect(LvnsInfo *lvns);

/* LvnsScript.c OP/ED  ץȽ */
int LvnsRunScript(LvnsInfo *lvns, LvnsScript *script);

/* LvnsImage.c  */
void LvnsLoadBackground(LvnsInfo *lvns, const char *basename, int no);
void LvnsClear(LvnsInfo *lvns, LvnsEffectType effect_clear);
void LvnsDisp(LvnsInfo *lvns, LvnsEffectType effect_disp);
void LvnsClearDisp(LvnsInfo *lvns,
                       LvnsEffectType effect_clear,
                       LvnsEffectType effect_disp);

/* LvnsFile.c եɤ߹ */
u_char *LvnsLoadData(LvnsInfo *lvns, const char *name, size_t *size);
LvnsImage *LvnsLoadImage(LvnsInfo *lvns, const char *name, LvnsImage *over);
void LvnsLoadScenario(LvnsInfo *lvns, int scn, int blk);
void LvnsLoadScenarioBlock(LvnsInfo *lvns, int blk);
void LvnsLoadScenarioText(LvnsInfo *lvns, int no);
u_char *LvnsGetScenarioText(LvnsInfo *lvns, int no);

/* LvnsEffect.c */
void LvnsInitClearEffect(LvnsInfo *lvns);
void LvnsInitDispEffect(LvnsInfo *lvns);
Bool LvnsDispEffect(LvnsInfo *lvns);
Bool LvnsClearEffect(LvnsInfo *lvns);
Bool LvnsDarken(LvnsInfo *lvns, int to_latitude, int diff);
Bool LvnsLighten(LvnsInfo *lvns, int to_latitude, int diff);
const char* LvnsEffectName(LvnsEffectType type);

/* LvnsEtc.c */
void LvnsInitSavePoint(LvnsInfo *lvns, struct SavePoint *sp);
void LvnsSetSavePoint(LvnsInfo *lvns, struct SavePoint *sp);

/* LvnsControl.c */
void LvnsMotion(LvnsInfo *lvns, int x, int y);
void LvnsClickLeft(LvnsInfo *lvns);
void LvnsClickRight(LvnsInfo *lvns);
void LvnsImageMode(LvnsInfo *lvns);
void LvnsSkipTillSelect(LvnsInfo *lvns);
void LvnsCursorUp(LvnsInfo *lvns);
void LvnsCursorDown(LvnsInfo *lvns);

/* ƥ¸ؿ */
void LvnsFlushWindow(LvnsInfo *lvns);
void LvnsFlushWindowArea(LvnsInfo *lvns, int x, int y, int w, int h);
void LvnsDrawWindow(LvnsInfo *lvns);
void LvnsDrawWindowArea(LvnsInfo *lvns, int x, int y, int w, int h, int x2, int y2);
void LvnsClearWindow(LvnsInfo *lvns);
void LvnsClearWindowArea(LvnsInfo *lvns, int x, int y, int w, int h);
void LvnsSetPaletteIndex(LvnsInfo *lvns, int idx, int r, int g, int b);
void LvnsSetPalette(LvnsInfo *lvns);
void LvnsSetPaletteMulti(LvnsInfo *lvns, int par16);
void LvnsSetPaletteScreen(LvnsInfo *lvns, int par16);
void LvnsDrawWindowMask(LvnsInfo *lvns, int x, int y, int state);
void LvnsDrawWindowSquareMask(LvnsInfo *lvns, int x, int y, int state);
void LvnsSetDispOffset(LvnsInfo *lvns, int xoff, int yoff, int maxoff);
void LvnsSyncWindow(LvnsInfo *lvns);

void LvnsPutPattern(LvnsInfo *lvns, int x, int y, int index, u_char *data);
void LvnsPutPattern2(LvnsInfo *lvns, int x, int y, int index, u_char *data);
void LvnsResetTimer(LvnsInfo *lvns, int no);
long LvnsGetTimer(LvnsInfo *lvns, int no);

void LvnsStartEngine(LvnsInfo *lvns);
void LvnsStopEngine(LvnsInfo *lvns);
void LvnsEnd(LvnsInfo *lvns);

void LvnsDrawBox(LvnsInfo *lvns, int x, int y, int w, int h, int idx);

time_t LvnsGetFileTime(LvnsInfo *lvns, const char *path);

/* Lvns.c */
void LvnsInitialize(LvnsInfo *lvns);
void LvnsDestroy(LvnsInfo *lvns);
void LvnsClearCursor(LvnsInfo *lvns);
void LvnsDrawCursor(LvnsInfo *lvns, LvnsCursorType cursor_type);
void LvnsClearTextCursor(LvnsInfo *lvns);
void LvnsDrawTextCursor(LvnsInfo *lvns);
void LvnsDrawMenu(LvnsInfo *lvns);

void LvnsDispWindow(LvnsInfo *lvns, int text_mode);

void LvnsInterval(LvnsInfo *lvns);
long LvnsMainEngine(LvnsInfo *lvns);
void LvnsStartSystem(LvnsInfo *lvns);
void LvnsSetState(LvnsInfo *lvns, enum LvnsState state);

void LvnsSetFunction(LvnsInfo *lvns, LvnsFunc func);

#endif

